@@ -30,7 +30,7 @@ sub new
   $$self{out} = [];
 
   $$self{source} = "";
-  $$self{tmpdir} = "/tmp";
+  $$self{tmpdir} = File::Spec->tmpdir();
   $$self{destdir} = "";
   $$self{objDefsParser} = undef; # It will be set in set_defsparser()
 
@@ -119,11 +119,9 @@ sub output_wrap_vfunc_h($$$$$$)
     $cppVfuncDecl .= " const";
   }
 
-  $self->append("#ifdef GLIBMM_VFUNCS_ENABLED\n");
   $self->ifdef($ifdef);
   $self->append("  $cppVfuncDecl;\n");
   $self->endif($ifdef);
-  $self->append("#endif //GLIBMM_VFUNCS_ENABLED\n");
 
   #The default callback, which will call *_vfunc, which will then call the base default callback.
   #Declares the callback in the private *Class class and sets it in the class_init function.
@@ -291,46 +289,9 @@ sub output_wrap_meth($$$$$$$)
   # Doxygen documentation before the method declaration:
   $self->output_wrap_meth_docs_only($filename, $line_num, $documentation);
 
- $self->ifdef($ifdef);
-
-  if($$objCDefsFunc{throw_any_errors})
-  {
-     $self->append("#ifdef GLIBMM_EXCEPTIONS_ENABLED\n");
-  }
+  $self->ifdef($ifdef);
 
   $self->append("  ${cppMethodDecl};");
-
-  if($$objCDefsFunc{throw_any_errors})
-  {
-     $self->append("\n#else\n");
-
-     # #Add an error argument, by searching for ) at the end and replacing it:
-     # my $declWithErrorArg = ${cppMethodDecl};
-     # $declWithErrorArg =~ s/\)$/, std::auto_ptr<Glib::Error>& error\)/g;
-
-     #Recreate the declaration, to remove the default values, which we can't have as well as an error parameter at the end: 
-     my $declWithErrorArg = $$objCppfunc{rettype} . " " . $$objCppfunc{name} . "(" . $objCppfunc->args_types_and_names() . ", std::auto_ptr<Glib::Error>& error)";
-
-     if($$objCppfunc{static})
-     {
-       $declWithErrorArg = "static " . $declWithErrorArg;
-     }
-
-     if($objCppfunc->get_is_const() eq 1)
-     {
-       if($$objCppfunc{static} ne 1) #It can't be const and static at the same time.
-       {
-         $declWithErrorArg = $declWithErrorArg . " const";
-       }
-     }
-
-     #remove any superfluous ,:
-     $declWithErrorArg =~ s/\(, /\(/g;
-
-     $self->append("  ${declWithErrorArg};");
-
-     $self->append("\n#endif //GLIBMM_EXCEPTIONS_ENABLED\n");
-  }
   
   $self->endif($ifdef);
 
@@ -447,15 +408,41 @@ sub output_wrap_create($$$)
 
 # void output_wrap_sig_decl($filename, $line_num, $objCSignal, $objCppfunc, $signal_name, $bCustomCCallback)
 # custom_signalproxy_name is "" when no type conversion is required - a normal templates SignalProxy will be used instead.
-sub output_wrap_sig_decl($$$$$$$)
+sub output_wrap_sig_decl($$$$$$$$)
 {
-  my ($self, $filename, $line_num, $objCSignal, $objCppfunc, $signal_name, $bCustomCCallback, $ifdef) = @_;
+  my ($self, $filename, $line_num, $objCSignal, $objCppfunc, $signal_name, $bCustomCCallback, $ifdef, $merge_doxycomment_with_previous) = @_;
 
 # _SIGNAL_PROXY(c_signal_name, c_return_type, `<c_arg_types_and_names>',
 #               cpp_signal_name, cpp_return_type, `<cpp_arg_types>',`<c_args_to_cpp>',
 #               refdoc_comment)
 
-  my $str = sprintf("_SIGNAL_PROXY(%s,%s,\`%s\',%s,%s,\`%s\',\`%s\',\`%s\',%s,%s)dnl\n",
+  my $doxycomment = $objCppfunc->get_refdoc_comment();
+
+  # If there was already a previous doxygen comment, we want to merge this
+  # one with the previous so it is one big comment. If it were two separate
+  # comments, doxygen would ignore the first one. If
+  # $merge_doxycomment_with_previous is nonzero, the first comment is
+  # already open but not yet closed.
+  if($merge_doxycomment_with_previous)
+  {
+    # Strip leading whitespace
+    $doxycomment =~ s/^\s+//;
+    # We don't have something to add, so just close the comment.
+    if($doxycomment eq "")
+    {
+      $doxycomment = "   */";
+    }
+    else
+    {
+      # Append the new comment, but remove the first three leading characters
+      # (which are /**) that mark the beginning of the comment.
+      $doxycomment = substr($doxycomment, 3);
+      $doxycomment =~ s/^\s+//;
+      $doxycomment = "   " . $doxycomment;
+    }
+  }
+
+  my $str = sprintf("_SIGNAL_PROXY(%s,%s,\`%s\',%s,%s,\`%s\',\`%s\',\`%s\',\`%s\',%s)dnl\n",
     $signal_name,
     $$objCSignal{rettype},
     $objCSignal->args_types_and_names_without_object(),
@@ -464,7 +451,7 @@ sub output_wrap_sig_decl($$$$$$$)
     $objCppfunc->args_types_only(),
     convert_args_c_to_cpp($objCSignal, $objCppfunc, $line_num),
     $bCustomCCallback, #When this is true, it will not write the *_callback implementation for you.
-    $objCppfunc->get_refdoc_comment(),
+    $doxycomment,
     $ifdef
   );
 
@@ -572,6 +559,12 @@ sub output_wrap_property($$$$$$)
     {
       # construct-only functions can be read, but not written.
       $proxy_suffix = "_ReadOnly";
+
+      if($objProperty->get_readable() ne 1)
+      {
+        $self->output_wrap_failed($name, "attempt to wrap write-only and construct-only property.");
+        return;
+      }
     }
     elsif($objProperty->get_readable() ne 1)
     {
@@ -584,9 +577,16 @@ sub output_wrap_property($$$$$$)
 
     # Convert - to _ so we can use it in C++ method and variable names:
     my $name_underscored = $name;
-    $name_underscored =~ s/-/_/g;
+    $name_underscored =~ tr/-/_/;
 
-    my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,%s)dnl\n",
+      # For the docs of the property (the final argument of the sprintf), the
+      # m4 quotes are changed, the new quotes are then used to quote the docs
+      # and then the quotes are changed back to the standard quotes.  This is
+      # done so that if there are commas in the docs, the contents after the
+      # docs are not lost (m4 thinks the contents after the comma is another
+      # agument to the macro).  Using the standard quotes leaves a trailing
+      # single quote for some undetermined reason.
+    my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,changequote([,])[%s]changequote(`,'))dnl\n",
       $name,
       $name_underscored,
       $cpp_type,
@@ -599,7 +599,15 @@ sub output_wrap_property($$$$$$)
     # If the property is not already read-only, and the property can be read, then add a second const accessor for a read-only propertyproxy:
     if( ($proxy_suffix ne "_ReadOnly") && ($objProperty->get_readable()) )
     {
-      my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,%s)dnl\n",
+
+      # For the docs of the property (the final argument of the sprintf), the
+      # m4 quotes are changed, the new quotes are then used to quote the docs
+      # and then the quotes are changed back to the standard quotes.  This is
+      # done so that if there are commas in the docs, the contents after the
+      # docs are not lost (m4 thinks the contents after the comma is another
+      # agument to the macro).  Using the standard quotes leaves a trailing
+      # single quote for some undetermined reason.
+      my $str = sprintf("_PROPERTY_PROXY(%s,%s,%s,%s,changequote([,])[%s]changequote(`,'))dnl\n",
         $name,
         $name_underscored,
         $cpp_type,
@@ -608,8 +616,6 @@ sub output_wrap_property($$$$$$)
       );
       $self->append($str);
     }
-
-    
   }
 }
 
@@ -636,7 +642,7 @@ sub make_g2_from_g1($)
   my ($self) = @_;
 
   # Execute m4 to get *.g2 file:
-  system("$$self{m4path} $$self{m4args} '$$self{tmpdir}/gtkmmproc_$$.g1' > '$$self{tmpdir}/gtkmmproc_$$.g2'");
+  system("$$self{m4path} $$self{m4args} \"$$self{tmpdir}/gtkmmproc_$$.g1\" > \"$$self{tmpdir}/gtkmmproc_$$.g2\"");
   return ($? >> 8);
 }
 
@@ -683,7 +689,12 @@ sub write_sections_to_files()
   foreach($fname_h, $fname_ph, $fname_cc)
   {
     # overwrite the source file only if it has actually changed
-    system("cmp -s '$_.tmp' '$_' || cp '$_.tmp' '$_' ; rm -f '$_.tmp'");
+
+    # Win32 does fail at this, so we do the two steps separately:
+    #system("cmp -s '$_.tmp' '$_' || cp '$_.tmp' '$_'" ; rm -f '$_.tmp');
+
+    system("cmp -s '$_.tmp' '$_' || cp '$_.tmp' '$_'");
+    system("rm -f '$_.tmp'");
   }
 }
 
